#include "mainwindow.h"
#include "ui_mainwindow.h"

#include "qfunc3d.h"
#include <QTextStream>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    //     
    ui->mainToolBar->addAction(ui->actionRestore);

    //  
    grLay = new QGridLayout(ui->frame);
    //  
    grLay->setContentsMargins(1,1,1,1);

    //  
    surf = new Qwt3D::SurfacePlot(ui->frame);
    //   
    surf->setTitle("SurfacePlot from Array");

    //     
    grLay->addWidget(surf,0,0);

    //       
    int N,M;
    //   Z  ,  N  M
    double *Zd = loadData("data.txt",&N,&M);
    //       X  Y
    double *Xd = (double *)malloc((N+M)*sizeof(double));
    double *Yd = Xd + N;

    // X    0   0.5
    for (int k=0; k < N; k++) Xd[k] = 0.5*k;
    // Y    -40   0.5
    for (int k=0; k < M; k++) Yd[k] = -40 + 0.5*k;

    //    ,   
    QFunc3D func(surf,Xd,Yd,Zd,N,M);
    //     (X,Y)
    func.setDomain(0,180,-40,40);
    //     
    func.setMesh(61,41);
//    func.setMesh(N,M);
    //    
    func.create();

    //   
    free((void *)Xd);
    free((void *)Zd);

    //    
    surf->setCoordinateStyle(Qwt3D::BOX);   // Qwt3D::FRAME
//    surf->setPlotStyle(Qwt3D::FILLED);
//    surf->setOrtho(false);

    //   
    QStringList labs;
    labs << "X" << "Y" << "Z";
    int num[12] = {0,1,2,0,0,0,1,1,1,2,2,2};
    for (int k=0; k != (int)surf->coordinates()->axes.size(); k++)
    {
        //  
        surf->coordinates()->axes[k].setLabelString(labs[num[k]]);
        //  
        surf->coordinates()->axes[k].setMajors(5);  // 
        surf->coordinates()->axes[k].setMinors(5);  // 
    }
    //  
    surf->coordinates()->adjustNumbers(6);  //    
    surf->coordinates()->adjustLabels(12);  //    

    //     
    rotateLab = new QLabel(ui->statusBar);  //  
    ui->statusBar->addWidget(rotateLab,0);
    shiftLab = new QLabel(ui->statusBar);   // 
    ui->statusBar->addWidget(shiftLab,0);
    scaleLab = new QLabel(ui->statusBar);   //   
    ui->statusBar->addWidget(scaleLab,0);
    zoomLab = new QLabel(ui->statusBar);    //  
    ui->statusBar->addWidget(zoomLab,0);

    //    
        //  
    connect(surf, SIGNAL(rotationChanged(double,double,double)), this, SLOT(procRotate(double,double,double)));
    connect(surf, SIGNAL(rotationChanged(double,double,double)), this, SLOT(showRotate(double,double,double)));
        //  
    connect(surf, SIGNAL(vieportShiftChanged(double,double)), this, SLOT(showShift(double,double)));
        //     
    connect(surf, SIGNAL(scaleChanged(double,double,double)), this, SLOT(showScale(double,double,double)));
        //   
    connect(surf, SIGNAL(zoomChanged(double)), this, SLOT(showZoom(double)));

    //    
    on_actionRestore_triggered();
    showShift(0,0);
}

MainWindow::~MainWindow()
{
    // 
    delete surf;    // 
    delete grLay;   // 

    delete ui;
}

void MainWindow::on_actionRestore_triggered()
{
    //   
    surf->setRotation(35,0,-20);    //  
    surf->setViewportShift(0,0);    // 
    surf->setScale(1,1.8,0.5);      //   
    surf->setZoom(0.95);            //  

    surf->updateData();
    surf->updateGL();
}

//     
void MainWindow::procRotate(double x,double y,double z)
{
    x *= M_PI/180;
    y *= M_PI/180;
    z *= M_PI/180;

    int gl = Qwt3D::NOSIDEGRID;

    double sp = cos(x)*sin(z) + sin(x)*sin(y)*cos(z);
    if (sp < 0) gl |= Qwt3D::LEFT;
    if (sp > 0) gl |= Qwt3D::RIGHT;

    sp = cos(x)*cos(z) - sin(x)*sin(y)*sin(z);
    if (sp < 0) gl |= Qwt3D::FRONT;
    if (sp > 0) gl |= Qwt3D::BACK;

    sp = -sin(x)*cos(y);
    if (sp < 0) gl |= Qwt3D::FLOOR;
    if (sp > 0) gl |= Qwt3D::CEIL;

    surf->coordinates()->setGridLines(true,false,gl);
}

void MainWindow::showRotate(double x,double y,double z) {
    rotateLab->setText(
        " Angles (" +
            QString::number(x,'g',3) + ", " +
            QString::number(y,'g',3) + ", " +
            QString::number(z,'g',3) + ")  ");
}

void MainWindow::showShift(double x,double y) {
    shiftLab->setText(
        " Shifts (" +
            QString::number(x,'g',3) + ", " +
            QString::number(y,'g',3) + ")  ");
}

void MainWindow::showScale(double x,double y,double z) {
    scaleLab->setText(
        " Scales (" +
            QString::number(x,'g',3) + ", " +
            QString::number(y,'g',3) + ", " +
            QString::number(z,'g',3) + ")  ");
}

void MainWindow::showZoom(double z) {
    zoomLab->setText(
        " Zoom "  +
            QString::number(z,'g',3) + "  ");
}

//     
double *MainWindow::loadData(QString fnm,int *Nx,int *My)
{
    *Nx = 0; *My = 0;
    QList<QList<double> *> td;
    QString dir = QApplication::applicationDirPath();
    //  
    QFile file(dir + "/" + fnm);
    if (!file.exists())
    {
        file.setFileName(dir + "/../" + fnm);
        if (!file.exists()) return NULL;
    }
    //       
    file.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream in(&file);      //   
    while (!in.atEnd()) //      
    {
        //  
        QString str = in.readLine(0);
        QStringList lst =str.split('\t');
        //  
        if (*Nx == 0) *Nx = lst.count();
        QList<double> *ld = new QList<double>();
        for (int n=0; n < *Nx; n++) ld->append(lst.at(n).toDouble());
        td.append(ld);
    }
    file.close();       //  
    //  
    *My = td.count();
    //      
    double *res = (double *)malloc((*My * *Nx) * sizeof(double));
    //      
    for (int m=0; m < *My; m++)
    {
        for (int n=0; n < *Nx; n++) res[m * *Nx + n] = td.at(m)->at(n);
        delete td.at(m);
    }
    td.clear();
    return res;
}
